//
//  BKFoundationProtocols.h
//  Blocks
//
//  Created by Jesse Grosjean on 4/1/06.
//  Copyright 2006 Hog Bay Software. All rights reserved.
//	2007-12-16	huperniketes	-NSString::extractNameFromContent: -	Changed type of parm to unsigned int to match NSString's -length method.
//	2009-09-10	huperniketes	Added macro definitions for Cocoa's message forwarding mechanism: prefixed by BKMessageForwarding_.
//
#import <Cocoa/Cocoa.h>
#import <Blocks/Blocks.h>

#pragma mark action proxy support

#define	BKMessageForwarding_methodSignatureForSelector(msgTarget)				\
- (NSMethodSignature *)methodSignatureForSelector: (SEL)anAction				\
{																				\
	NSMethodSignature	*methodSignature;										\
																				\
	methodSignature = [super methodSignatureForSelector: anAction];				\
	if((methodSignature == nil) && (msgTarget != nil))							\
		methodSignature = [msgTarget methodSignatureForSelector: anAction];		\
	return	(methodSignature);													\
}

#define	BKMessageForwarding_forwardInvocation(msgTarget)						\
- (void)forwardInvocation: (NSInvocation *)anInvocation							\
{																				\
	SEL		invocationSelector = [anInvocation selector];						\
																				\
	if((msgTarget != nil) && [msgTarget respondsToSelector: invocationSelector]) \
		[anInvocation invokeWithTarget: msgTarget];								\
	else																		\
		[super forwardInvocation: anInvocation];								\
}

#define	BKMessageForwarding_respondsToSelector(msgTarget)						\
- (BOOL)respondsToSelector: (SEL)anAction										\
{																				\
	BOOL	respondsToSelector;													\
																				\
	respondsToSelector = [super respondsToSelector: anAction];					\
	if((respondsToSelector != YES) && (msgTarget != nil))						\
		respondsToSelector = [msgTarget respondsToSelector: anAction];			\
	return	(respondsToSelector);												\
}

#define	BKMessageForwarding_MinimumSet(msgTarget)								\
BKMessageForwarding_methodSignatureForSelector(msgTarget)						\
BKMessageForwarding_forwardInvocation(msgTarget)								\
BKMessageForwarding_respondsToSelector(msgTarget)

typedef enum _BKDateIntervalType {
    BKDayInterval = 1,
    BKWeekInterval = 2,
    BKMonthInterval = 3,
    BKYearInterval = 4,
} BKDateIntervalType;

@protocol	BKUniqueStringApproval

- (BOOL)string: (NSString *)aString isUniqueInContext: (id)anObject;

@end

@interface NSString (BKFoundationAdditions)

+ (NSString *)uniqueString;
- (NSString *)md5Digest;
- (NSString *)sha1Digest;
- (NSString *)stringByEscapingEntities;
- (NSString *)stringByUnescapingEntities;
- (NSString *)stringByEscapingFilenameBackslashes;
- (NSString *)stringByUnescapingFilenameBackslashes;
- (NSString *)utiFromPath;
- (NSArray *)extensionsAndOSTypesFromUTI;
- (NSString *)extractNameFromContent: (unsigned int)maxLength;
- (unsigned int)wordCountInRange:(NSRange)range paragraphCount:(unsigned int *)paragraphCount;
- (NSArray *)componentsSeparatedByLineSeparators;
//- (NSString *)stringFromTextClipping:(NSString *)textClippingPath;
- (NSString *)uniqueCopyApprovedBy: (id <BKUniqueStringApproval>)anObject inContext: (id)aContext;
- (NSString *)uniqueCopyWithExtensionApprovedBy: (id <BKUniqueStringApproval>)anObject inContext: (id)aContext;

@end

@interface NSArray (BKFoundationAdditions)

- (id)firstObject;
- (NSMutableArray *)arrayByFlattening;
- (unsigned)insertIndexForObject:(id)object withSortDescriptors:(NSArray *)sortDescriptors;
- (NSArray *)arrayByRemovingObject:(id)object;
- (NSArray *)arrayByRemovingObjectsInArray: (NSArray *)aSubtractorArray;
- (NSMutableSet *)setByFlattening;
- (NSArray *)filePathsToFileURLs;

@end

#define	NSAttributedString_SetAttribute(s, a, v)		[s addAttribute: a value: v range: NSMakeRange(0, [s length])]
#define	NSAttributedString_SetForegroundColor(s, c)		NSAttributedString_SetAttribute(s, NSForegroundColorAttributeName, c)

@interface NSAttributedString (BKAttributedStringAdditions)

+ (NSAttributedString *)readStringAtURL:(NSURL *)url error:(NSError **)error;

@end

@interface NSMutableArray (BKFoundationAdditions)

- (void)insertObject:(id)newObject after:(id)object;
- (void)insertObject:(id)newObject before:(id)object;
- (void)fillToLength:(unsigned)length with:(id)object;
- (BOOL)enqueueObject:(id)object;
- (id)dequeueObject;

@end

@interface NSDictionary (BKFoundationAdditions)

- (BOOL)boolForKey:(id)key;
- (NSRect)rectForKey:(id)key;
- (float)floatForKey:(id)key;
- (int)intForKey:(id)key;
- (NSSize)sizeForKey:(id)key;

@end

@interface NSMutableDictionary (BKFoundationAdditions)

- (void)setBool:(BOOL)boolean forKey:(id)key;
- (void)setRect:(NSRect)rect forKey:(id)key;
- (void)setFloat:(float)value forKey:(id)key;
- (void)setInt:(int)value forKey:(id)key;
- (void)setSize:(NSSize)size forKey:(id)key;

@end

@interface NSEnumerator (BKFoundationAdditions)

+ (NSEnumerator *)emptyEnumerator;
- (id)objectAtIndex:(unsigned int)index;
- (unsigned int)indexOfObject:(id)object;

@end

@interface NSDate (BKFoundationAdditions)

- (NSCalendarDate *)startOf:(BKDateIntervalType)intervalType;
- (NSCalendarDate *)endOf:(BKDateIntervalType)intervalType;
- (NSCalendarDate *)back:(int)interval intervalType:(BKDateIntervalType)intervalType;
- (NSCalendarDate *)forward:(int)interval intervalType:(BKDateIntervalType)intervalType;

@end

@interface NSURL (BKFoundationAdditions)

- (NSDictionary *)parameters;
- (NSURL *)URLWithParameters:(NSDictionary *)newParameters;

@end

@interface NSDateFormatter (BKFoundationAdditions)

+ (NSDate *)dateFromString:(NSString *)string;
- (NSString *)osx10_0CompatibleDataFormat;

@end

@interface NSNumberFormatter (BKFoundationAdditions)

+ (NSNumber *)numberFromString:(NSString *)string;
+ (NSString *)stringFromNumber:(NSNumber *)number;

@end

@interface NSFileManager (BKFoundationAdditions)

- (NSString *)uniqueTempPath;
- (NSString *)uniqueFilePathForDirectory:(NSString *)directory preferedName:(NSString *)name;
- (NSString *)processesApplicationSupportFolder;
- (NSString *)findSystemFolderType:(int)folderType forDomain:(int)domain;
- (BOOL)isPathOnWebDAVFileSystem:(NSString *)path;

@end

@interface		NSObject	(BKFoundationAdditions)

- (BOOL)isKindOfCollection;

@end

@interface NSPredicate (BKFoundationAdditions)

+ (NSDictionary *)defaultSubstitutionVariables;
+ (NSPredicate *)predicate:(NSPredicate *)filterPredicate includingObjects:(NSArray *)objects;
+ (NSPredicate *)predicate:(NSPredicate *)filterPredicate excludingObjects:(NSArray *)objects;

@end

@interface NSFetchRequest (BKFoundationAdditions)

+ (void)prefetch:(NSArray *)managedObjects entityName:(NSString *)entityName managedObjectContext:(NSManagedObjectContext *)managedObjectContext;

@end

@interface NSEntityDescription (BKFoundationAdditions)

- (NSPropertyDescription *)propertyDescriptionForKeyPath:(NSString *)keyPath;

@end

@interface NSSet (BKFoundationAdditions)

- (NSMutableArray *)arrayByFlattening;
- (NSMutableSet *)setByFlattening;

@end

#define NSPredicateStartOfDaySubstitutionVariable @"NSPredicateStartOfDaySubstitutionVariable"
#define NSPredicateStartOfDaySubstitutionVariable @"NSPredicateStartOfDaySubstitutionVariable"
#define NSPredicateEndOfDaySubstitutionVariable @"NSPredicateEndOfDaySubstitutionVariable"
#define NSPredicateStartOfWeekSubstitutionVariable @"NSPredicateStartOfWeekSubstitutionVariable"
#define NSPredicateEndOfWeekSubstitutionVariable @"NSPredicateEndOfWeekSubstitutionVariable"
#define NSPredicateStartOfLastWeekSubstitutionVariable @"NSPredicateStartOfLastWeekSubstitutionVariable"
#define NSPredicateEndOfLastWeekSubstitutionVariable @"NSPredicateEndOfLastWeekSubstitutionVariable"
#define NSPredicateStartOfMonthSubstitutionVariable @"NSPredicateStartOfMonthSubstitutionVariable"
#define NSPredicateEndOfMonthSubstitutionVariable @"NSPredicateEndOfMonthSubstitutionVariable"
#define NSPredicateStartOfYearSubstitutionVariable @"NSPredicateStartOfYearSubstitutionVariable"
#define NSPredicateEndOfYearSubstitutionVariable @"NSPredicateEndOfYearSubstitutionVariable"
