/*
 *	BKTreeNode.h
 *	Blocks
 *
 *	© Copyright 2011 Apokalypse Software Corp. All rights reserved.
 *
 *	2011-10-16	huperniketes	Created.
 */
#import <Cocoa/Cocoa.h>

@class NSArray, NSMutableArray, NSIndexPath;

@interface	BKTreeNode : NSObject	{
	id					representedObject;
	id					childNodesProxy;
	void				*observationInfo;
	NSMutableArray		*childNodes;
	BKTreeNode			*parentNode;	//not retained
	NSString			*keyPathToChildren;
}

+ (id)treeNodeWithRepresentedObject: (id)aModelObject;
- (id)initWithRepresentedObject: (id)aModelObject;

- (id)representedObject;
- (NSString *)childrenKeyPath;
- (void)setChildrenKeyPath: (NSString *)aKeyPath;

    // stats
- (NSIndexPath *)indexPath; // represents the receivers location in entire tree
- (BOOL)isLeaf; // determined by (count of child nodes == 0)
    
- (NSArray *)childNodes;    // traversal
- (NSMutableArray *)mutableChildNodes; // returns a mutable proxy - parentNode of inserted/removed child nodes is automatically updated
- (BKTreeNode *)descendantNodeAtIndexPath: (NSIndexPath *)anIndexPath; // traversal begins with receiver

- (BKTreeNode *)parentNode;

    // sorts the entire subtree
- (void)sortWithSortDescriptors: (NSArray *)anArrayOfSortDescriptors recursively: (BOOL)aRecurseFlag;

@end
