/*
 *  BKLicenseProtocols.h
 *  Blocks
 *
 *	Created by Alfonso Guerra on 2/1/09.
 *	Copyright 2009 Apokalypse Software Corp.. All rights reserved.
 *
 */
#import <Cocoa/Cocoa.h>

@protocol BKLicenseProtocol <NSObject>

#if		defined(POSTPONED_UNTIL_NEEDED)

#pragma mark class methods

+ (BOOL)removeLicense:(NSString *)licenseName;
+ (BOOL)isLicenseValid:(NSString *)licenseName;
+ (BOOL)isLicenseValidAtPath:(NSString *)path publicKey:(NSData *)publicKey;
+ (BKLicense *)licenseNamed:(NSString *)licenseName;
+ (void)uncacheLicenseNamed:(NSString *)licenseName;
+ (void)dontShowDonateWindowForLicenseNamed:(NSString *)name;
+ (BOOL)showsDonateWindowForLicenseNamed:(NSString *)name;

#pragma mark init

- (id)initLicenseWithContentsOfFile:(NSString *)path;

#pragma mark accessors

- (NSData *)publicKey;
- (void)setPublicKey:(NSData *)publicKey;
- (NSData *)privateKey;
- (void)setPrivateKey:(NSData *)privateKey;
- (NSString *)licenseKey;
- (void)setLicenseKey:(NSString *)licenseKey;

#pragma mark license data

- (NSString *)licenseName;
- (void)setLicenseName:(NSString *)licenseName;
- (NSString *)firstName;
- (void)setFirstName:(NSString *)firstName;
- (NSString *)lastName;
- (void)setLastName:(NSString *)lastName;
- (NSString *)email;
- (void)setEmail:(NSString *)email;
- (NSString *)numberOfUsers;
- (void)setNumberOfUsers:(NSString *)numberOfUsers;
- (NSString *)orderNumber;
- (void)setOrderNumber:(NSString *)orderNumber;
- (NSString *)licenseInfoAsString;
- (NSString *)licenseInfoForKey:(NSString *)key;
- (void)setLicenseInfo:(NSString *)string forKey:(NSString *)key;

#pragma mark behaviors

- (BOOL)writeToFile:(NSString *)path;
- (NSString *)generateLicenseKey;

#endif

#pragma mark behaviors

- (BOOL)isValid;

@end

@protocol BKLicenseControllerProtocol <NSObject>

#if	defined(POSTPONED_UNTIL_NEEDED)

#pragma mark accessors

- (BKLicense *)license;
- (BOOL)valid;

#pragma mark actions

- (IBAction)upgradeDonation:(id)sender;
- (IBAction)buyNow:(id)sender;

#endif

#pragma mark class methods

+ (id)sharedInstance;

@end

@interface NSApplication (BKLicenseControllerAccess)
- (id <BKLicenseControllerProtocol>)BKLicenseProtocols_licenseController; // exposed this way for applescript support
@end

